<?php

namespace Drupal\d8_sitemap\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Url;
use Symfony\Component\HttpFoundation\Response;

class D8SitemapController extends ControllerBase {

  public function getSitemap() {
    $xsl_url = Url::fromRoute('d8_sitemap.xsl')->toString();
    $query = \Drupal::database()->query('SELECT xml FROM d8_sitemap WHERE xml IS NOT NULL');
    $xml = '<?xml version="1.0" encoding="UTF-8"?>
<?xml-stylesheet type="text/xsl" href="' . $xsl_url . '"?>
<!--Generated by the D8 Sitemap Drupal module. -->
<urlset xmlns="http://www.sitemaps.org/schemas/sitemap/0.9" xmlns:xhtml="http://www.w3.org/1999/xhtml" xmlns:image="http://www.google.com/schemas/sitemap-image/1.1">';
    foreach ($query as $row) {
      $xml .= $row->xml;
    }
    $xml .= '</urlset>';
    \Drupal::service('page_cache_kill_switch')->trigger();
    return new Response($xml, 200, [
      'Content-type' => 'text/xml',
      'Cache-Control' => 'public',
      'Expires' => gmdate('D, d M Y H:i:s \G\M\T', time() + 86400),
    ]);
  }

  /**
   * Returns the XML stylesheet for the sitemap.
   *
   * @return \Symfony\Component\HttpFoundation\Response
   */
  public function getXsl() {
    $path = drupal_get_path('module', 'd8_sitemap');
    $xsl_content = file_get_contents($path . '/xsl/simple_sitemap.xsl');

    // Replace custom tokens in the XSL content with appropriate values.
    $replacements = [
      '[title]' => 'Sitemap file',
      '[generated-by]' => 'Generated by the D8Sitemap Drupal module',
      '[number-of-sitemaps]' => 'Number of sitemaps in this index',
      '[sitemap-url]' => 'Sitemap URL',
      '[number-of-urls]' => 'Number of URLs in this sitemap',
      '[url-location]' => 'URL location',
      '[lastmod]' => 'Last modification date',
      '[changefreq]' => 'Change frequency',
      '[priority]' => 'Priority',
      '[translation-set]' => 'Translation set',
      '[images]' => 'Images',
      '[image-title]' => 'Title',
      '[image-caption]' => 'Caption',
      '[jquery]' => base_path() . 'core/assets/vendor/jquery/jquery.min.js',
      '[jquery-tablesorter]' => base_path() . $path . '/xsl/jquery.tablesorter.min.js',
      '[parser-date-iso8601]' => base_path() . $path . '/xsl/parser-date-iso8601.min.js',
      '[xsl-js]' => base_path() . $path . '/xsl/simple_sitemap.xsl.js',
      '[xsl-css]' => base_path() . $path . '/xsl/simple_sitemap.xsl.css',
    ];

    // Output the XSL content.
    return new Response(strtr($xsl_content, $replacements), Response::HTTP_OK, [
      'Content-type' => 'application/xml; charset=utf-8',
      'X-Robots-Tag' => 'noindex, nofollow',
    ]);
  }

}
